# DragonLog (c) 2023-2025 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/

import json
import socket
import logging
import time

from PyQt6 import QtWidgets, QtCore, QtGui
from PyQt6.QtCore import QModelIndex
from PyQt6.QtGui import QStandardItem

from . import DxSpots_ui
from .Logger import Logger
from .cty import CountryNotFoundException, CountryCodeNotFoundException


class DxCluster(QtCore.QThread):
    spotReceived = QtCore.pyqtSignal(str)
    auroraBeaconReceived = QtCore.pyqtSignal(str)
    announcementReceived = QtCore.pyqtSignal(str)
    dataReceived = QtCore.pyqtSignal(str)
    newMailReceived = QtCore.pyqtSignal(str)

    def __init__(self, logger: Logger, address: str, port: int, call: str):
        super().__init__()

        self.log = logging.getLogger('DxCluster')
        self.log.addHandler(logger)
        self.log.setLevel(logger.loglevel)
        self.logger = logger
        self.log.debug('Initialising...')

        self.__address__ = address, port
        self.__callsign__ = call.upper()
        self.__receive__ = True
        self.__socket__ = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.__is_connected__ = False
        self.__connect__()
        self.__cmds__: list = []
        self.__lock__ = QtCore.QMutex()

        self.dataReceived.connect(self.logData)
        self.newMailReceived.connect(self.logMessage)
        self.announcementReceived.connect(self.logMessage)
        self.auroraBeaconReceived.connect(self.logAuroraBeacon)

    def __connect__(self):
        self.__socket__.connect(self.__address__)
        self.__socket__.settimeout(.1)
        data = ''
        login_rcvd = False
        t_start = time.time()
        while not login_rcvd and time.time() - t_start < 2000:
            try:
                data += self.__socket__.recv(1024).replace(b'\r', b'').decode()
                if data.endswith('login: '):
                    login_rcvd = True
            except TimeoutError:
                pass

        if login_rcvd:
            self.log.debug('Received login request')
            self.__socket__.sendall(f'{self.__callsign__}\n'.encode())

            try:
                data = self.__socket__.recv(1024).decode()
                if data.startswith('Hello'):
                    self.__is_connected__ = True
                    self.log.info(
                        f'Loggedin to DX cluster {self.__address__[0]}:{self.__address__[1]} as {self.__callsign__}')
                    for dt in data.split('\n'):
                        if dt.startswith('New mail has arrived for you'):
                            self.newMailReceived.emit(dt.strip())
                            break
                elif data.startswith('Sorry'):
                    raise Exception(f'Login error with call "{self.__callsign__}": valid callsign required')
                else:
                    self.log.debug(data)
                    raise Exception(f'Login failed to DX cluster {self.__address__[0]}:{self.__address__[1]}')
            except TimeoutError:
                pass
        else:
            raise Exception(f'Connection failed to DX cluster {self.__address__[0]}:{self.__address__[1]}')

    @property
    def isConnected(self) -> bool:
        return self.__is_connected__

    def stop(self):
        self.sendCmd('bye')
        time.sleep(.1)
        self.__receive__ = False
        self.__is_connected__ = False
        time.sleep(.1)
        self.__socket__.close()
        self.log.debug('Diconnected from cluster')

    def run(self):
        self.__socket__.sendall(b'show/dx 10 real\nset/dx\n')
        while self.__receive__:
            try:
                data = self.__socket__.recv(1024).replace(b'\r', b'').decode()
                for dt in reversed(data.strip().split('\n')):
                    if dt.startswith('DX de'):
                        # DX de S52WW:     14268.0  SK6BA        USB                            0636Z
                        self.spotReceived.emit(dt.strip())
                    elif dt.startswith('WCY de'):
                        # WCY de DK0WCY-2 <08> : K=1 expK=0 A=22 R=95 SFI=214 SA=act GMF=act Au=no
                        self.auroraBeaconReceived.emit(dt.strip())
                    # elif dt.startswith('WWV de'):
                    # WWV de AE5E <09>:   SFI=214, A=20, K=1, Minor w/G1 -> No Storms
                    # self.auroraBeaconReceived.emit(dt.strip())
                    elif dt.startswith('To ALL de'):
                        # To ALL de ON5CAZ: Onff-0189 wca on-02734 pota be-0239
                        self.announcementReceived.emit(dt.strip())
                    elif dt.startswith('New mail has arrived for you'):
                        self.newMailReceived.emit(dt.strip())
                    else:
                        self.dataReceived.emit(dt.strip())
                if self.__cmds__:
                    self.__lock__.lock()
                    cmd = self.__cmds__.pop(0)
                    self.__socket__.sendall(f'{cmd}\n'.encode())
                    self.__lock__.unlock()
            except TimeoutError:
                pass
            except ConnectionAbortedError:
                self.__receive__ = False
            except UnicodeDecodeError:
                pass

    def logData(self, data: str):
        self.log.debug(data.replace('\a', ''))

    def logMessage(self, data: str):
        self.log.warning(data.replace('\a', ''))

    def logAuroraBeacon(self, data: str):
        self.log.info(data.replace('\a', ''))

    def sendCmd(self, cmd: str):
        self.__lock__.lock()
        self.__cmds__.append(cmd)
        self.__lock__.unlock()


# noinspection PyPep8Naming
class DxSpotsFilterProxy(QtCore.QSortFilterProxyModel):
    def __init__(self, parent, columns: int):
        super().__init__(parent)

        self.columns = columns
        self.filter: dict[int, str] | None = None
        self.clearFilter()

    def clearFilter(self):
        self.filter = dict(enumerate([''] * self.columns))

    def setFilter(self, column: int, filter: str):
        self.filter[column] = filter
        self.invalidateFilter()

    def filterAcceptsRow(self, sourceRow, sourceParent):
        indexes = [self.sourceModel().index(sourceRow, i, sourceParent) for i in range(self.columns)]

        return all([self.filter[i] == '' or self.sourceModel().data(indexes[i]) == self.filter[i] for i in
                    range(self.columns)])


class FlagsTableModel(QtGui.QStandardItemModel):
    """Show flags in country column"""

    def __init__(self, parent, dragonlog, country_col: int, call_cols: tuple[int, int]):
        super(FlagsTableModel, self).__init__(parent)

        self.country_col = country_col
        self.call_cols = call_cols

        self.countries: dict = {}
        with open(dragonlog.searchFile(f'icons:flags/flags_map.json')) as fm_f:
            self.countries: dict = json.load(fm_f)

        for c in self.countries:
            self.countries[c] = QtGui.QIcon(dragonlog.searchFile(f'icons:flags/{self.countries[c]}.png'))

    def data(self, idx, role=QtCore.Qt.ItemDataRole.DisplayRole):
        value = super().data(idx, role)

        if role == QtCore.Qt.ItemDataRole.DecorationRole:
            if idx.column() == self.country_col:
                txt = super().data(idx,
                                   QtCore.Qt.ItemDataRole.DisplayRole).replace('&', 'and').replace('St. ', 'Saint ')
                if txt in self.countries:
                    return self.countries[txt]

        return value


class DxSpots(QtWidgets.QDialog, DxSpots_ui.Ui_DxSpotsForm):
    spotSelected = QtCore.pyqtSignal(str, str, float)

    def __init__(self, parent, dragonlog, settings: QtCore.QSettings, logger: Logger):
        super().__init__(parent)

        self.setupUi(self)

        self.log = logging.getLogger('DxSpots')
        self.log.addHandler(logger)
        self.log.setLevel(logger.loglevel)
        self.logger = logger
        self.log.debug('Initialising...')

        self.__dragonlog__ = dragonlog
        self.__settings__ = settings

        self.__refresh_timer__ = QtCore.QTimer(self)

        self.header = [
            self.tr('Spotter'),
            self.tr('Sp.Cnt.'),
            self.tr('Freq.'),
            self.tr('DX Call'),
            self.tr('Comments'),
            self.tr('Time'),
            self.tr('Cont.'),
            self.tr('Band'),
            self.tr('Country'),
        ]

        self.tableModel = FlagsTableModel(self, dragonlog, 8, (0, 3))
        self.filterModel = DxSpotsFilterProxy(self, len(self.header))
        self.filterModel.setSourceModel(self.tableModel)
        self.tableView.setModel(self.filterModel)

        self.clear()

        bands: list = self.__settings__.value('ui/show_bands', list(dragonlog.bands.keys()))
        if '11m' in bands:
            bands.remove('11m')
        self.bandComboBox.insertItem(0, self.tr('- all -'))
        self.bandComboBox.insertItems(1, bands)

        self.spContComboBox.insertItems(0, [self.tr('- all -'), 'AF', 'AN', 'AS', 'EU', 'NA', 'OC', 'SA'])

        self.__refresh__ = False

        self.dxc = None

    def clear(self):
        self.tableModel.clear()
        self.tableModel.setHorizontalHeaderLabels(self.header)
        self.tableView.resizeColumnsToContents()

    def control(self, state: bool):
        if state:
            try:
                self.__refresh__ = True
                dx_call = self.__settings__.value('dx_spots/call', '')
                self.dxc = DxCluster(self.logger,
                                     self.__settings__.value('dx_spots/address', 'hamqth.com'),
                                     int(self.__settings__.value('dx_spots/port', 7300)),
                                     dx_call if dx_call else self.__settings__.value('station/callSign', ''))
                self.clear()
                self.dxc.spotReceived.connect(self.processSpot)
                self.dxc.start()
                self.startPushButton.setText(self.tr('Stop'))
            except Exception as exc:
                self.log.error(str(exc))
                self.startPushButton.setChecked(False)
                QtWidgets.QMessageBox.warning(self, self.tr('DX Spot'),
                                              self.tr('Error connecting to DX Cluster'))
        else:
            self.__refresh__ = False
            if self.dxc:
                self.dxc.stop()
            self.startPushButton.setText(self.tr('Start'))

    def band_from_freq(self, freq: float) -> str:
        """Get band from frequenzy
        :param freq: frequency in kHz
        :return: band"""
        freq = float(freq)
        for b in self.__dragonlog__.bands:
            if freq < self.__dragonlog__.bands[b][1]:
                if freq > self.__dragonlog__.bands[b][0]:
                    return b
        return ''

    def processSpot(self, data: str):
        self.log.debug(data)
        if len(data) < 74 and not data.startswith('DX de'):
            return

        spot = [''] * 9
        spot[1] = '-'
        spot[6] = '-'
        spot[8] = self.tr('- unknown -')

        try:
            spotter, freq = data[6:24].split(':')
            call = data[26:39].strip()
            comment = data[39:70].strip()
            tstamp = f'{data[70:72]}:{data[72:74]}'

            spot[0] = spotter
            spot[2] = freq.strip()
            spot[3] = call
            spot[4] = comment
            spot[5] = tstamp
            spot[7] = self.band_from_freq(float(freq.strip()))

            sp_cty = self.__dragonlog__.cty_data(spotter)
            if sp_cty:
                spot[1] = sp_cty.continent

            cty_d = self.__dragonlog__.cty_data(call)
            if cty_d:
                spot[6] = cty_d.continent
                spot[8] = cty_d.name
        except (CountryCodeNotFoundException, CountryNotFoundException) as exc:
            self.log.error(exc)
        except Exception as exc:
            self.log.exception(exc)

        self.addSpotToTable(spot)

    def addSpotToTable(self, data: list):
        self.tableModel.insertRow(0, [QStandardItem(d) for d in data])
        self.tableView.resizeColumnsToContents()

        if self.tableModel.rowCount() > self.nrSpotsSpinBox.value():
            self.tableModel.removeRow(self.tableModel.rowCount() - 1)

    def bandChanged(self, band: str):
        if not band.startswith('-'):
            self.filterModel.setFilter(7, band)
        else:
            self.filterModel.setFilter(7, '')

    def spContChanged(self, cont: str):
        if not cont.startswith('-'):
            self.filterModel.setFilter(1, cont)
        else:
            self.filterModel.setFilter(1, '')

    def selectSpot(self, index: QModelIndex):
        call = index.siblingAtColumn(3).data()
        band = index.siblingAtColumn(7).data()
        try:
            freq = float(index.siblingAtColumn(2).data())
        except ValueError:
            freq = 0.0
        self.log.debug(f'Selected spot {call}, {band}, {freq}')
        self.spotSelected.emit(call, band, freq)



# DragonLog (c) 2025 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/
"""Contains other contests"""

from .base import (ContestLogCBR, CBRRecord, Address, BandStatistics, BAND_FROM_CBR, ExchangeData,
                   CategoryMode, CategoryBand, CategoryPower, CategoryOperator, CategoryAssisted, CategoryTransmitter)
from dragonlog.cty import CountryData, Country, CountryCodeNotFoundException


class IARUHFWorldChampionshipContest(ContestLogCBR):
    contest_name = 'IARU HF World Championship'
    contest_year = '2025'
    contest_update = '2025-07-08'
    contest_exch_fmt = 'ITU-Zone'

    def __init__(self, callsign: str, name: str, club: str, address: Address, email: str, locator: str,
                 band: type[CategoryBand], mode: type[CategoryMode],
                 pwr: type[CategoryPower] = CategoryPower.HIGH,
                 cat_operator: type[CategoryOperator] = CategoryOperator.SINGLE_OP,
                 assisted: type[CategoryAssisted] = CategoryAssisted.NON_ASSISTED,
                 tx: type[CategoryTransmitter] = CategoryTransmitter.ONE,
                 operators: list[str] = None, specific: str = '', skip_id: bool = False,
                 skip_warn: bool = False, logger=None, **params):
        super().__init__(callsign, name, club, address, email, locator,
                         band, mode, pwr, cat_operator,
                         assisted, tx, operators, specific, skip_id, skip_warn, logger)

        self.cty: CountryData | None = params.get('cty', None)
        if not self.cty or type(self.cty) is not CountryData:
            self.cty = None
            self.log.error('Error with cty data')

        try:
            self.own_cty_data: Country = self.cty.country(callsign)
        except CountryCodeNotFoundException:
            self.log.error(f'Country data could not be found for your callsign "{callsign}"')
            self.own_cty_data: Country = Country(*([''] * 9))

        self.__multis_band__: dict[str, set] = dict(zip(self.valid_bands_list()[1:],
                                                        [set() for _ in range(len(self.valid_bands_list()[1:]))]))
        self.__multis2_band__: dict[str, set] = dict(zip(self.valid_bands_list()[1:],
                                                         [set() for _ in range(len(self.valid_bands_list()[1:]))]))

    def build_record(self, adif_rec) -> CBRRecord:
        adif_rec['STX_STRING'] = self.own_cty_data.itu
        rec = super().build_record(adif_rec)
        return rec

    def process_points(self, rec: CBRRecord):
        try:
            band = BAND_FROM_CBR[rec.band]
            if band not in self.__stats__:
                self.error(f'Wrong band "{band}" for contest')
                return

            call_cty = self.cty.country(rec.call)
            qso_point = 1
            self.__rated__ += 1

            if rec.rcvd_exch.strip().isdigit():
                self.__multis__.add(rec.rcvd_exch.strip())
                self.__multis_band__[band].add(rec.rcvd_exch.strip())

                if rec.rcvd_exch.strip() != self.own_cty_data.itu:
                    qso_point = 3
                    if call_cty.continent != self.own_cty_data.continent:
                        qso_point = 5
            else:
                self.__multis2__.add(rec.rcvd_exch.strip().upper())
                self.__multis2_band__[band].add(rec.rcvd_exch.strip().upper())

            self.__points__ += qso_point

            # fill stats
            self.__stats__[band].qsos += 1
            self.__stats__[band].rated += 1
            self.__stats__[band].points += qso_point
            self.__stats__[band].multis = len(self.__multis_band__[band])
            self.__stats__[band].multis2 = len(self.__multis2_band__[band])

            self.__header__['CLAIMED-SCORE'] = str(self.claimed_points)
        except Exception:
            self.exception()

    @property
    def contest_id(self) -> str:
        return 'IARU-HF'

    @property
    def statistics(self) -> dict[str, BandStatistics]:
        stats = self.__stats__.copy()
        qsos = 0
        rated = 0
        points = 0
        multis = 0
        multis2 = 0
        for b in stats:
            stats[b].summary = stats[b].points * (
                    stats[b].multis + stats[b].multis2)
            qsos += stats[b].qsos
            rated += stats[b].rated
            points += stats[b].points
            multis += stats[b].multis
            multis2 += stats[b].multis2
        stats['Total'] = BandStatistics(qsos, rated, points, multis, multis2,
                                        points * (multis + multis2))
        return stats

    @classmethod
    def statistic_primer(cls) -> BandStatistics:
        return BandStatistics(multis=0)

    @property
    def claimed_points(self) -> int:
        """Points with some kind of multiplicator"""
        stat = self.statistics['Total']
        return stat.points * (stat.multis + stat.multis2)

    def summary(self) -> str:
        """A summary of points for the contest"""
        stat = self.statistics['Total']
        return (f"QSOs: {self.qsos}, Rated: {self.rated}, Points: {stat.points}, "
                f"Multis: {stat.multis}, Multis2: {stat.multis2}, Claimed points: {stat.summary}")

    @property
    def file_name(self) -> str:
        return f'{self.__contest_date__}_{self.__header__["CALLSIGN"]}-IARU_HF_Championship.cbr'

    @classmethod
    def valid_modes(cls) -> tuple[CategoryMode, ...]:
        return CategoryMode.SSB, CategoryMode.MIXED, CategoryMode.CW

    @classmethod
    def valid_bands(cls) -> tuple[CategoryBand, ...]:
        return (CategoryBand.B_ALL, CategoryBand.B_160M, CategoryBand.B_80M, CategoryBand.B_40M,
                CategoryBand.B_20M, CategoryBand.B_15M, CategoryBand.B_10M)

    @classmethod
    def is_single_day(cls) -> bool:
        return False

    @classmethod
    def valid_power(cls) -> tuple[CategoryPower, ...]:
        return CategoryPower.HIGH, CategoryPower.LOW, CategoryPower.QRP

    @classmethod
    def valid_operator(cls) -> tuple[CategoryOperator, ...]:
        return CategoryOperator.SINGLE_OP, CategoryOperator.CHECKLOG

    @staticmethod
    def extract_exchange(exchange: str) -> ExchangeData | None:
        if exchange:
            return ExchangeData(itu_zone=exchange.strip())
        else:
            return None

    @staticmethod
    def prepare_exchange(exchange: ExchangeData):
        return f'{exchange.itu_zone}'



# Form implementation generated from reading ui file 'DragonLog_Settings.ui'
#
# Created by: PyQt6 UI code generator 6.9.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(450, 550)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Dialog.sizePolicy().hasHeightForWidth())
        Dialog.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(parent=Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setMinimumSize(QtCore.QSize(0, 450))
        self.tabWidget.setObjectName("tabWidget")
        self.station = QtWidgets.QWidget()
        self.station.setObjectName("station")
        self.formLayout = QtWidgets.QFormLayout(self.station)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(parent=self.station)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label)
        self.callsignLineEdit = QtWidgets.QLineEdit(parent=self.station)
        self.callsignLineEdit.setObjectName("callsignLineEdit")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.callsignLineEdit)
        self.label_2 = QtWidgets.QLabel(parent=self.station)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_2)
        self.nameLineEdit = QtWidgets.QLineEdit(parent=self.station)
        self.nameLineEdit.setObjectName("nameLineEdit")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.nameLineEdit)
        self.label_3 = QtWidgets.QLabel(parent=self.station)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_3)
        spacerItem = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Minimum)
        self.formLayout.setItem(3, QtWidgets.QFormLayout.ItemRole.FieldRole, spacerItem)
        self.label_5 = QtWidgets.QLabel(parent=self.station)
        self.label_5.setObjectName("label_5")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_5)
        self.radioComboBox = QtWidgets.QComboBox(parent=self.station)
        self.radioComboBox.setObjectName("radioComboBox")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.radioComboBox)
        self.label_6 = QtWidgets.QLabel(parent=self.station)
        self.label_6.setObjectName("label_6")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_6)
        self.antennaComboBox = QtWidgets.QComboBox(parent=self.station)
        self.antennaComboBox.setObjectName("antennaComboBox")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.antennaComboBox)
        spacerItem1 = QtWidgets.QSpacerItem(20, 500, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.formLayout.setItem(6, QtWidgets.QFormLayout.ItemRole.FieldRole, spacerItem1)
        self.qthComboBox = QtWidgets.QComboBox(parent=self.station)
        self.qthComboBox.setObjectName("qthComboBox")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.qthComboBox)
        self.tabWidget.addTab(self.station, "")
        self.listings = QtWidgets.QWidget()
        self.listings.setObjectName("listings")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout(self.listings)
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.listingsWidget = QtWidgets.QWidget(parent=self.listings)
        self.listingsWidget.setObjectName("listingsWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.listingsWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout_12.addWidget(self.listingsWidget)
        self.tabWidget.addTab(self.listings, "")
        self.bands_modes = QtWidgets.QWidget()
        self.bands_modes.setObjectName("bands_modes")
        self.verticalLayout_14 = QtWidgets.QVBoxLayout(self.bands_modes)
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.bandsGroupBox = QtWidgets.QGroupBox(parent=self.bands_modes)
        self.bandsGroupBox.setObjectName("bandsGroupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.bandsGroupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_14.addWidget(self.bandsGroupBox)
        self.modesGroupBox = QtWidgets.QGroupBox(parent=self.bands_modes)
        self.modesGroupBox.setObjectName("modesGroupBox")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout(self.modesGroupBox)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.verticalLayout_14.addWidget(self.modesGroupBox)
        self.tabWidget.addTab(self.bands_modes, "")
        self.cb_station = QtWidgets.QWidget()
        self.cb_station.setObjectName("cb_station")
        self.formLayout_2 = QtWidgets.QFormLayout(self.cb_station)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_10 = QtWidgets.QLabel(parent=self.cb_station)
        self.label_10.setObjectName("label_10")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_10)
        self.callsignCBLineEdit = QtWidgets.QLineEdit(parent=self.cb_station)
        self.callsignCBLineEdit.setObjectName("callsignCBLineEdit")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.callsignCBLineEdit)
        self.label_9 = QtWidgets.QLabel(parent=self.cb_station)
        self.label_9.setObjectName("label_9")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_9)
        self.radioCBLineEdit = QtWidgets.QLineEdit(parent=self.cb_station)
        self.radioCBLineEdit.setObjectName("radioCBLineEdit")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.radioCBLineEdit)
        self.label_11 = QtWidgets.QLabel(parent=self.cb_station)
        self.label_11.setObjectName("label_11")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_11)
        self.antennaCBLineEdit = QtWidgets.QLineEdit(parent=self.cb_station)
        self.antennaCBLineEdit.setObjectName("antennaCBLineEdit")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.antennaCBLineEdit)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.formLayout_2.setItem(5, QtWidgets.QFormLayout.ItemRole.FieldRole, spacerItem2)
        self.cbDefaultCheckBox = QtWidgets.QCheckBox(parent=self.cb_station)
        self.cbDefaultCheckBox.setObjectName("cbDefaultCheckBox")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.cbDefaultCheckBox)
        self.expCBQSOsCheckBox = QtWidgets.QCheckBox(parent=self.cb_station)
        self.expCBQSOsCheckBox.setObjectName("expCBQSOsCheckBox")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.ItemRole.FieldRole, self.expCBQSOsCheckBox)
        self.tabWidget.addTab(self.cb_station, "")
        self.cat = QtWidgets.QWidget()
        self.cat.setObjectName("cat")
        self.formLayout_3 = QtWidgets.QFormLayout(self.cat)
        self.formLayout_3.setObjectName("formLayout_3")
        self.rigctldPathLabel = QtWidgets.QLabel(parent=self.cat)
        self.rigctldPathLabel.setObjectName("rigctldPathLabel")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.rigctldPathLabel)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.hamlibPathLineEdit = QtWidgets.QLineEdit(parent=self.cat)
        self.hamlibPathLineEdit.setReadOnly(True)
        self.hamlibPathLineEdit.setObjectName("hamlibPathLineEdit")
        self.horizontalLayout_2.addWidget(self.hamlibPathLineEdit)
        self.hamlibPathToolButton = QtWidgets.QToolButton(parent=self.cat)
        self.hamlibPathToolButton.setObjectName("hamlibPathToolButton")
        self.horizontalLayout_2.addWidget(self.hamlibPathToolButton)
        self.formLayout_3.setLayout(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.horizontalLayout_2)
        self.checkHamlibLabel = QtWidgets.QLabel(parent=self.cat)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 0, 0))
        brush.setStyle(QtCore.Qt.BrushStyle.SolidPattern)
        palette.setBrush(QtGui.QPalette.ColorGroup.Active, QtGui.QPalette.ColorRole.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 0, 0))
        brush.setStyle(QtCore.Qt.BrushStyle.SolidPattern)
        palette.setBrush(QtGui.QPalette.ColorGroup.Inactive, QtGui.QPalette.ColorRole.WindowText, brush)
        self.checkHamlibLabel.setPalette(palette)
        self.checkHamlibLabel.setText("")
        self.checkHamlibLabel.setObjectName("checkHamlibLabel")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.checkHamlibLabel)
        self.label_12 = QtWidgets.QLabel(parent=self.cat)
        self.label_12.setObjectName("label_12")
        self.formLayout_3.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_12)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.catInterfaceLineEdit = QtWidgets.QLineEdit(parent=self.cat)
        self.catInterfaceLineEdit.setObjectName("catInterfaceLineEdit")
        self.horizontalLayout_5.addWidget(self.catInterfaceLineEdit)
        self.catBaudComboBox = QtWidgets.QComboBox(parent=self.cat)
        self.catBaudComboBox.setObjectName("catBaudComboBox")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.catBaudComboBox.addItem("")
        self.horizontalLayout_5.addWidget(self.catBaudComboBox)
        self.formLayout_3.setLayout(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.horizontalLayout_5)
        self.label_13 = QtWidgets.QLabel(parent=self.cat)
        self.label_13.setObjectName("label_13")
        self.formLayout_3.setWidget(5, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_13)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.manufacturerComboBox = QtWidgets.QComboBox(parent=self.cat)
        self.manufacturerComboBox.setObjectName("manufacturerComboBox")
        self.horizontalLayout_3.addWidget(self.manufacturerComboBox)
        self.modelComboBox = QtWidgets.QComboBox(parent=self.cat)
        self.modelComboBox.setObjectName("modelComboBox")
        self.horizontalLayout_3.addWidget(self.modelComboBox)
        self.formLayout_3.setLayout(5, QtWidgets.QFormLayout.ItemRole.FieldRole, self.horizontalLayout_3)
        spacerItem3 = QtWidgets.QSpacerItem(20, 1000, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.formLayout_3.setItem(6, QtWidgets.QFormLayout.ItemRole.FieldRole, spacerItem3)
        self.checkHamlibRunLabel = QtWidgets.QLabel(parent=self.cat)
        self.checkHamlibRunLabel.setText("")
        self.checkHamlibRunLabel.setObjectName("checkHamlibRunLabel")
        self.formLayout_3.setWidget(7, QtWidgets.QFormLayout.ItemRole.FieldRole, self.checkHamlibRunLabel)
        self.tabWidget.addTab(self.cat, "")
        self.userinterface = QtWidgets.QWidget()
        self.userinterface.setObjectName("userinterface")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.userinterface)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.groupBox_12 = QtWidgets.QGroupBox(parent=self.userinterface)
        self.groupBox_12.setObjectName("groupBox_12")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout(self.groupBox_12)
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout()
        self.verticalLayout_16.setContentsMargins(-1, 4, -1, -1)
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.fontComboBox = QtWidgets.QFontComboBox(parent=self.groupBox_12)
        self.fontComboBox.setObjectName("fontComboBox")
        self.verticalLayout_16.addWidget(self.fontComboBox)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_16.addItem(spacerItem4)
        self.horizontalLayout_12.addLayout(self.verticalLayout_16)
        self.formLayout_8 = QtWidgets.QFormLayout()
        self.formLayout_8.setObjectName("formLayout_8")
        self.label_14 = QtWidgets.QLabel(parent=self.groupBox_12)
        self.label_14.setObjectName("label_14")
        self.formLayout_8.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_14)
        self.fontSizeSpinBox = QtWidgets.QSpinBox(parent=self.groupBox_12)
        self.fontSizeSpinBox.setMaximumSize(QtCore.QSize(60, 16777215))
        self.fontSizeSpinBox.setMinimum(6)
        self.fontSizeSpinBox.setMaximum(48)
        self.fontSizeSpinBox.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.fontSizeSpinBox.setProperty("value", 9)
        self.fontSizeSpinBox.setObjectName("fontSizeSpinBox")
        self.formLayout_8.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.fontSizeSpinBox)
        self.label_15 = QtWidgets.QLabel(parent=self.groupBox_12)
        self.label_15.setObjectName("label_15")
        self.formLayout_8.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_15)
        self.qfFontSizeSpinBox = QtWidgets.QSpinBox(parent=self.groupBox_12)
        self.qfFontSizeSpinBox.setMaximumSize(QtCore.QSize(60, 16777215))
        self.qfFontSizeSpinBox.setMinimum(6)
        self.qfFontSizeSpinBox.setMaximum(48)
        self.qfFontSizeSpinBox.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.qfFontSizeSpinBox.setProperty("value", 9)
        self.qfFontSizeSpinBox.setObjectName("qfFontSizeSpinBox")
        self.formLayout_8.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.qfFontSizeSpinBox)
        self.label_25 = QtWidgets.QLabel(parent=self.groupBox_12)
        self.label_25.setObjectName("label_25")
        self.formLayout_8.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_25)
        self.ccFontSizeSpinBox = QtWidgets.QSpinBox(parent=self.groupBox_12)
        self.ccFontSizeSpinBox.setMaximumSize(QtCore.QSize(60, 16777215))
        self.ccFontSizeSpinBox.setMinimum(6)
        self.ccFontSizeSpinBox.setMaximum(48)
        self.ccFontSizeSpinBox.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.ccFontSizeSpinBox.setProperty("value", 9)
        self.ccFontSizeSpinBox.setObjectName("ccFontSizeSpinBox")
        self.formLayout_8.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.ccFontSizeSpinBox)
        self.horizontalLayout_12.addLayout(self.formLayout_8)
        self.verticalLayout_5.addWidget(self.groupBox_12)
        self.groupBox_2 = QtWidgets.QGroupBox(parent=self.userinterface)
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.sortComboBox = QtWidgets.QComboBox(parent=self.groupBox_2)
        self.sortComboBox.setObjectName("sortComboBox")
        self.verticalLayout_11.addWidget(self.sortComboBox)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_11.addItem(spacerItem5)
        self.horizontalLayout_8.addLayout(self.verticalLayout_11)
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.sortAscRadioButton = QtWidgets.QRadioButton(parent=self.groupBox_2)
        self.sortAscRadioButton.setChecked(True)
        self.sortAscRadioButton.setObjectName("sortAscRadioButton")
        self.verticalLayout_10.addWidget(self.sortAscRadioButton)
        self.sortDscRadioButton = QtWidgets.QRadioButton(parent=self.groupBox_2)
        self.sortDscRadioButton.setObjectName("sortDscRadioButton")
        self.verticalLayout_10.addWidget(self.sortDscRadioButton)
        self.horizontalLayout_8.addLayout(self.verticalLayout_10)
        self.verticalLayout_5.addWidget(self.groupBox_2)
        self.groupBox_7 = QtWidgets.QGroupBox(parent=self.userinterface)
        self.groupBox_7.setObjectName("groupBox_7")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.groupBox_7)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_20 = QtWidgets.QLabel(parent=self.groupBox_7)
        self.label_20.setObjectName("label_20")
        self.horizontalLayout_9.addWidget(self.label_20)
        self.recentQSOsComboBox = QtWidgets.QComboBox(parent=self.groupBox_7)
        self.recentQSOsComboBox.setObjectName("recentQSOsComboBox")
        self.recentQSOsComboBox.addItem("")
        self.recentQSOsComboBox.addItem("")
        self.recentQSOsComboBox.addItem("")
        self.recentQSOsComboBox.addItem("")
        self.recentQSOsComboBox.addItem("")
        self.horizontalLayout_9.addWidget(self.recentQSOsComboBox)
        spacerItem6 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_9.addItem(spacerItem6)
        self.verticalLayout_5.addWidget(self.groupBox_7)
        self.columnsGroupBox = QtWidgets.QGroupBox(parent=self.userinterface)
        self.columnsGroupBox.setObjectName("columnsGroupBox")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.columnsGroupBox)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalLayout_5.addWidget(self.columnsGroupBox)
        self.checkUpdatesCheckBox = QtWidgets.QCheckBox(parent=self.userinterface)
        self.checkUpdatesCheckBox.setChecked(True)
        self.checkUpdatesCheckBox.setObjectName("checkUpdatesCheckBox")
        self.verticalLayout_5.addWidget(self.checkUpdatesCheckBox)
        self.groupBox_8 = QtWidgets.QGroupBox(parent=self.userinterface)
        self.groupBox_8.setObjectName("groupBox_8")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.groupBox_8)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_21 = QtWidgets.QLabel(parent=self.groupBox_8)
        self.label_21.setObjectName("label_21")
        self.horizontalLayout_10.addWidget(self.label_21)
        self.logLevelComboBox = QtWidgets.QComboBox(parent=self.groupBox_8)
        self.logLevelComboBox.setCurrentText("Info")
        self.logLevelComboBox.setObjectName("logLevelComboBox")
        self.logLevelComboBox.addItem("")
        self.logLevelComboBox.setItemText(0, "Debug")
        self.logLevelComboBox.addItem("")
        self.logLevelComboBox.setItemText(1, "Info")
        self.logLevelComboBox.addItem("")
        self.logLevelComboBox.setItemText(2, "Warning")
        self.logLevelComboBox.addItem("")
        self.logLevelComboBox.setItemText(3, "Error")
        self.logLevelComboBox.addItem("")
        self.logLevelComboBox.setItemText(4, "Critical")
        self.horizontalLayout_10.addWidget(self.logLevelComboBox)
        self.logToFileCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_8)
        self.logToFileCheckBox.setObjectName("logToFileCheckBox")
        self.horizontalLayout_10.addWidget(self.logToFileCheckBox)
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem7)
        self.verticalLayout_5.addWidget(self.groupBox_8)
        spacerItem8 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_5.addItem(spacerItem8)
        self.horizontalLayout_6.addLayout(self.verticalLayout_5)
        self.tabWidget.addTab(self.userinterface, "")
        self.dxspots = QtWidgets.QWidget()
        self.dxspots.setObjectName("dxspots")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.dxspots)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox = QtWidgets.QGroupBox(parent=self.dxspots)
        self.groupBox.setObjectName("groupBox")
        self.formLayout_7 = QtWidgets.QFormLayout(self.groupBox)
        self.formLayout_7.setObjectName("formLayout_7")
        self.label_4 = QtWidgets.QLabel(parent=self.groupBox)
        self.label_4.setObjectName("label_4")
        self.formLayout_7.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_4)
        self.dxAddrLineEdit = QtWidgets.QLineEdit(parent=self.groupBox)
        self.dxAddrLineEdit.setObjectName("dxAddrLineEdit")
        self.formLayout_7.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.dxAddrLineEdit)
        self.label_7 = QtWidgets.QLabel(parent=self.groupBox)
        self.label_7.setObjectName("label_7")
        self.formLayout_7.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_7)
        self.dxPortSpinBox = QtWidgets.QSpinBox(parent=self.groupBox)
        self.dxPortSpinBox.setMinimum(1)
        self.dxPortSpinBox.setMaximum(65535)
        self.dxPortSpinBox.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.dxPortSpinBox.setProperty("value", 7300)
        self.dxPortSpinBox.setObjectName("dxPortSpinBox")
        self.formLayout_7.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.dxPortSpinBox)
        self.label_8 = QtWidgets.QLabel(parent=self.groupBox)
        self.label_8.setObjectName("label_8")
        self.formLayout_7.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_8)
        self.dxCallLineEdit = QtWidgets.QLineEdit(parent=self.groupBox)
        self.dxCallLineEdit.setObjectName("dxCallLineEdit")
        self.formLayout_7.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.dxCallLineEdit)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.groupBox_11 = QtWidgets.QGroupBox(parent=self.dxspots)
        self.groupBox_11.setObjectName("groupBox_11")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout(self.groupBox_11)
        self.horizontalLayout_13.setSpacing(0)
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.ctyPathLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_11)
        self.ctyPathLineEdit.setObjectName("ctyPathLineEdit")
        self.horizontalLayout_13.addWidget(self.ctyPathLineEdit)
        self.ctyPathToolButton = QtWidgets.QToolButton(parent=self.groupBox_11)
        self.ctyPathToolButton.setObjectName("ctyPathToolButton")
        self.horizontalLayout_13.addWidget(self.ctyPathToolButton)
        self.verticalLayout_3.addWidget(self.groupBox_11)
        spacerItem9 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_3.addItem(spacerItem9)
        self.tabWidget.addTab(self.dxspots, "")
        self.import_export = QtWidgets.QWidget()
        self.import_export.setObjectName("import_export")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.import_export)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox_3 = QtWidgets.QGroupBox(parent=self.import_export)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.expOwnNotesADIFCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_3)
        self.expOwnNotesADIFCheckBox.setObjectName("expOwnNotesADIFCheckBox")
        self.verticalLayout_7.addWidget(self.expOwnNotesADIFCheckBox)
        self.expRecentOnlyCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_3)
        self.expRecentOnlyCheckBox.setObjectName("expRecentOnlyCheckBox")
        self.verticalLayout_7.addWidget(self.expRecentOnlyCheckBox)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_26 = QtWidgets.QLabel(parent=self.groupBox_3)
        self.label_26.setObjectName("label_26")
        self.horizontalLayout_4.addWidget(self.label_26)
        self.csvDelimiterComboBox = QtWidgets.QComboBox(parent=self.groupBox_3)
        self.csvDelimiterComboBox.setObjectName("csvDelimiterComboBox")
        self.csvDelimiterComboBox.addItem("")
        self.csvDelimiterComboBox.addItem("")
        self.horizontalLayout_4.addWidget(self.csvDelimiterComboBox)
        spacerItem10 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem10)
        self.verticalLayout_7.addLayout(self.horizontalLayout_4)
        self.verticalLayout_2.addWidget(self.groupBox_3)
        self.groupBox_9 = QtWidgets.QGroupBox(parent=self.import_export)
        self.groupBox_9.setObjectName("groupBox_9")
        self.verticalLayout_13 = QtWidgets.QVBoxLayout(self.groupBox_9)
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.useCfgIDImportCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_9)
        self.useCfgIDImportCheckBox.setObjectName("useCfgIDImportCheckBox")
        self.verticalLayout_13.addWidget(self.useCfgIDImportCheckBox)
        self.useCfgStationImportCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_9)
        self.useCfgStationImportCheckBox.setObjectName("useCfgStationImportCheckBox")
        self.verticalLayout_13.addWidget(self.useCfgStationImportCheckBox)
        self.verticalLayout_2.addWidget(self.groupBox_9)
        self.groupBox_4 = QtWidgets.QGroupBox(parent=self.import_export)
        self.groupBox_4.setObjectName("groupBox_4")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.useCfgIDWatchCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_4)
        self.useCfgIDWatchCheckBox.setObjectName("useCfgIDWatchCheckBox")
        self.verticalLayout_8.addWidget(self.useCfgIDWatchCheckBox)
        self.useCfgStationWatchCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_4)
        self.useCfgStationWatchCheckBox.setObjectName("useCfgStationWatchCheckBox")
        self.verticalLayout_8.addWidget(self.useCfgStationWatchCheckBox)
        self.verticalLayout_2.addWidget(self.groupBox_4)
        self.groupBox_10 = QtWidgets.QGroupBox(parent=self.import_export)
        self.groupBox_10.setObjectName("groupBox_10")
        self.verticalLayout_15 = QtWidgets.QVBoxLayout(self.groupBox_10)
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.useCfgStationCCCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_10)
        self.useCfgStationCCCheckBox.setObjectName("useCfgStationCCCheckBox")
        self.verticalLayout_15.addWidget(self.useCfgStationCCCheckBox)
        self.verticalLayout_2.addWidget(self.groupBox_10)
        spacerItem11 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_2.addItem(spacerItem11)
        self.tabWidget.addTab(self.import_export, "")
        self.credentials = QtWidgets.QWidget()
        self.credentials.setObjectName("credentials")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.credentials)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.groupBox_5 = QtWidgets.QGroupBox(parent=self.credentials)
        self.groupBox_5.setObjectName("groupBox_5")
        self.formLayout_4 = QtWidgets.QFormLayout(self.groupBox_5)
        self.formLayout_4.setObjectName("formLayout_4")
        self.label_16 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_16.setObjectName("label_16")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_16)
        self.callbookUserLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_5)
        self.callbookUserLineEdit.setObjectName("callbookUserLineEdit")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.callbookUserLineEdit)
        self.label_17 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_17.setObjectName("label_17")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_17)
        self.callbookPasswdLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_5)
        self.callbookPasswdLineEdit.setEchoMode(QtWidgets.QLineEdit.EchoMode.PasswordEchoOnEdit)
        self.callbookPasswdLineEdit.setObjectName("callbookPasswdLineEdit")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.callbookPasswdLineEdit)
        self.callbookComboBox = QtWidgets.QComboBox(parent=self.groupBox_5)
        self.callbookComboBox.setObjectName("callbookComboBox")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.callbookComboBox)
        self.label_24 = QtWidgets.QLabel(parent=self.groupBox_5)
        self.label_24.setObjectName("label_24")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_24)
        self.verticalLayout_9.addWidget(self.groupBox_5)
        self.groupBox_6 = QtWidgets.QGroupBox(parent=self.credentials)
        self.groupBox_6.setObjectName("groupBox_6")
        self.formLayout_5 = QtWidgets.QFormLayout(self.groupBox_6)
        self.formLayout_5.setObjectName("formLayout_5")
        self.label_18 = QtWidgets.QLabel(parent=self.groupBox_6)
        self.label_18.setObjectName("label_18")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_18)
        self.eqslUserLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_6)
        self.eqslUserLineEdit.setObjectName("eqslUserLineEdit")
        self.formLayout_5.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.eqslUserLineEdit)
        self.label_19 = QtWidgets.QLabel(parent=self.groupBox_6)
        self.label_19.setObjectName("label_19")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_19)
        self.eqslPasswdLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_6)
        self.eqslPasswdLineEdit.setEchoMode(QtWidgets.QLineEdit.EchoMode.PasswordEchoOnEdit)
        self.eqslPasswdLineEdit.setObjectName("eqslPasswdLineEdit")
        self.formLayout_5.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.eqslPasswdLineEdit)
        self.verticalLayout_9.addWidget(self.groupBox_6)
        self.groupBox_81 = QtWidgets.QGroupBox(parent=self.credentials)
        self.groupBox_81.setObjectName("groupBox_81")
        self.formLayout_6 = QtWidgets.QFormLayout(self.groupBox_81)
        self.formLayout_6.setObjectName("formLayout_6")
        self.label_22 = QtWidgets.QLabel(parent=self.groupBox_81)
        self.label_22.setObjectName("label_22")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_22)
        self.lotwUserLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_81)
        self.lotwUserLineEdit.setObjectName("lotwUserLineEdit")
        self.formLayout_6.setWidget(0, QtWidgets.QFormLayout.ItemRole.FieldRole, self.lotwUserLineEdit)
        self.label_211 = QtWidgets.QLabel(parent=self.groupBox_81)
        self.label_211.setObjectName("label_211")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_211)
        self.lotwPasswdLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_81)
        self.lotwPasswdLineEdit.setEchoMode(QtWidgets.QLineEdit.EchoMode.PasswordEchoOnEdit)
        self.lotwPasswdLineEdit.setObjectName("lotwPasswdLineEdit")
        self.formLayout_6.setWidget(1, QtWidgets.QFormLayout.ItemRole.FieldRole, self.lotwPasswdLineEdit)
        self.lotwCertPwdCheckBox = QtWidgets.QCheckBox(parent=self.groupBox_81)
        self.lotwCertPwdCheckBox.setObjectName("lotwCertPwdCheckBox")
        self.formLayout_6.setWidget(3, QtWidgets.QFormLayout.ItemRole.FieldRole, self.lotwCertPwdCheckBox)
        self.label_23 = QtWidgets.QLabel(parent=self.groupBox_81)
        self.label_23.setObjectName("label_23")
        self.formLayout_6.setWidget(3, QtWidgets.QFormLayout.ItemRole.LabelRole, self.label_23)
        self.line = QtWidgets.QFrame(parent=self.groupBox_81)
        self.line.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.formLayout_6.setWidget(2, QtWidgets.QFormLayout.ItemRole.FieldRole, self.line)
        self.verticalLayout_9.addWidget(self.groupBox_81)
        spacerItem12 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout_9.addItem(spacerItem12)
        self.tabWidget.addTab(self.credentials, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(parent=Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        self.tabWidget.setCurrentIndex(0)
        self.logLevelComboBox.setCurrentIndex(1)
        self.buttonBox.accepted.connect(Dialog.accept) # type: ignore
        self.buttonBox.rejected.connect(Dialog.reject) # type: ignore
        self.hamlibPathToolButton.clicked.connect(Dialog.chooseHamlibPath) # type: ignore
        self.manufacturerComboBox.currentTextChanged['QString'].connect(Dialog.mfrChanged) # type: ignore
        self.callsignCBLineEdit.textChanged['QString'].connect(Dialog.callSignCBChanged) # type: ignore
        self.callsignLineEdit.textChanged['QString'].connect(Dialog.callSignChanged) # type: ignore
        self.callsignCBLineEdit.textEdited['QString'].connect(Dialog.callSignCBChanged) # type: ignore
        self.callsignLineEdit.textEdited['QString'].connect(Dialog.callSignChanged) # type: ignore
        self.callbookComboBox.currentTextChanged['QString'].connect(Dialog.callbookSelected) # type: ignore
        self.ctyPathToolButton.clicked.connect(Dialog.chooseCtyDataFile) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Settings"))
        self.label.setText(_translate("Dialog", "Call sign"))
        self.label_2.setText(_translate("Dialog", "Name"))
        self.label_3.setText(_translate("Dialog", "QTH (Locator)"))
        self.label_5.setText(_translate("Dialog", "Radio"))
        self.label_6.setText(_translate("Dialog", "Antenna"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.station), _translate("Dialog", "Station"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.listings), _translate("Dialog", "Listings"))
        self.bandsGroupBox.setTitle(_translate("Dialog", "Bands"))
        self.modesGroupBox.setTitle(_translate("Dialog", "Modes"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.bands_modes), _translate("Dialog", "Bands && Modes"))
        self.label_10.setText(_translate("Dialog", "Call sign"))
        self.label_9.setText(_translate("Dialog", "Radio"))
        self.label_11.setText(_translate("Dialog", "Antenna"))
        self.cbDefaultCheckBox.setText(_translate("Dialog", "Select CB band by default"))
        self.expCBQSOsCheckBox.setToolTip(_translate("Dialog", "On import CB QSOs will always be handled"))
        self.expCBQSOsCheckBox.setText(_translate("Dialog", "Export CB QSOs to ADIF"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.cb_station), _translate("Dialog", "CB-Station"))
        self.rigctldPathLabel.setText(_translate("Dialog", "Hamlib rigctld"))
        self.hamlibPathToolButton.setText(_translate("Dialog", "..."))
        self.label_12.setText(_translate("Dialog", "Interface"))
        self.catBaudComboBox.setItemText(0, _translate("Dialog", "128000"))
        self.catBaudComboBox.setItemText(1, _translate("Dialog", "115200"))
        self.catBaudComboBox.setItemText(2, _translate("Dialog", "57600"))
        self.catBaudComboBox.setItemText(3, _translate("Dialog", "38400"))
        self.catBaudComboBox.setItemText(4, _translate("Dialog", "19200"))
        self.catBaudComboBox.setItemText(5, _translate("Dialog", "14400"))
        self.catBaudComboBox.setItemText(6, _translate("Dialog", "9600"))
        self.catBaudComboBox.setItemText(7, _translate("Dialog", "4800"))
        self.catBaudComboBox.setItemText(8, _translate("Dialog", "2400"))
        self.catBaudComboBox.setItemText(9, _translate("Dialog", "1200"))
        self.label_13.setText(_translate("Dialog", "Rig"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.cat), _translate("Dialog", "CAT"))
        self.groupBox_12.setTitle(_translate("Dialog", "Font"))
        self.label_14.setText(_translate("Dialog", "Application font size"))
        self.fontSizeSpinBox.setSuffix(_translate("Dialog", " pt"))
        self.label_15.setText(_translate("Dialog", "QSO form font size"))
        self.qfFontSizeSpinBox.setSuffix(_translate("Dialog", " pt"))
        self.label_25.setText(_translate("Dialog", "Cassiopeia font size"))
        self.ccFontSizeSpinBox.setSuffix(_translate("Dialog", " pt"))
        self.groupBox_2.setTitle(_translate("Dialog", "Sort on column"))
        self.sortAscRadioButton.setText(_translate("Dialog", "Ascending"))
        self.sortDscRadioButton.setText(_translate("Dialog", "Descending"))
        self.groupBox_7.setTitle(_translate("Dialog", "Recent QSOs"))
        self.label_20.setText(_translate("Dialog", "Show QSOs from"))
        self.recentQSOsComboBox.setItemText(0, _translate("Dialog", "Show all"))
        self.recentQSOsComboBox.setItemText(1, _translate("Dialog", "Last week"))
        self.recentQSOsComboBox.setItemText(2, _translate("Dialog", "Last month"))
        self.recentQSOsComboBox.setItemText(3, _translate("Dialog", "Last half year"))
        self.recentQSOsComboBox.setItemText(4, _translate("Dialog", "Last year"))
        self.columnsGroupBox.setTitle(_translate("Dialog", "Show or hide columns"))
        self.checkUpdatesCheckBox.setText(_translate("Dialog", "Check for updates at startup"))
        self.groupBox_8.setToolTip(_translate("Dialog", "Effective after application restart"))
        self.groupBox_8.setTitle(_translate("Dialog", "Logging output"))
        self.label_21.setText(_translate("Dialog", "Level"))
        self.logToFileCheckBox.setText(_translate("Dialog", "Log to file"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.userinterface), _translate("Dialog", "User interface"))
        self.groupBox.setTitle(_translate("Dialog", "DX Cluster"))
        self.label_4.setText(_translate("Dialog", "Address"))
        self.dxAddrLineEdit.setText(_translate("Dialog", "hamqth.com"))
        self.label_7.setText(_translate("Dialog", "Port"))
        self.label_8.setText(_translate("Dialog", "Call"))
        self.groupBox_11.setTitle(_translate("Dialog", "Country Data"))
        self.ctyPathToolButton.setText(_translate("Dialog", "..."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.dxspots), _translate("Dialog", "Dx Spots"))
        self.groupBox_3.setTitle(_translate("Dialog", "Export"))
        self.expOwnNotesADIFCheckBox.setText(_translate("Dialog", "Export own notes to ADIF"))
        self.expRecentOnlyCheckBox.setToolTip(_translate("Dialog", "See settings page \"User interface\""))
        self.expRecentOnlyCheckBox.setText(_translate("Dialog", "Export only recent QSOs"))
        self.label_26.setText(_translate("Dialog", "CSV delimiter"))
        self.csvDelimiterComboBox.setItemText(0, _translate("Dialog", ","))
        self.csvDelimiterComboBox.setItemText(1, _translate("Dialog", ";"))
        self.groupBox_9.setTitle(_translate("Dialog", "ADIF Import"))
        self.useCfgIDImportCheckBox.setText(_translate("Dialog", "Use configured ID for missing values"))
        self.useCfgStationImportCheckBox.setText(_translate("Dialog", "Use configured Station for missing values"))
        self.groupBox_4.setTitle(_translate("Dialog", "Watch File"))
        self.useCfgIDWatchCheckBox.setText(_translate("Dialog", "Use configured ID for missing values"))
        self.useCfgStationWatchCheckBox.setText(_translate("Dialog", "Use configured Station for missing values"))
        self.groupBox_10.setTitle(_translate("Dialog", "CassiopeiaConsole"))
        self.useCfgStationCCCheckBox.setText(_translate("Dialog", "Use configured Station for missing values"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.import_export), _translate("Dialog", "Import/Export"))
        self.groupBox_5.setTitle(_translate("Dialog", "Callbook"))
        self.label_16.setText(_translate("Dialog", "Username"))
        self.label_17.setText(_translate("Dialog", "Password"))
        self.label_24.setText(_translate("Dialog", "Service"))
        self.groupBox_6.setTitle(_translate("Dialog", "eQSL"))
        self.label_18.setText(_translate("Dialog", "Username"))
        self.label_19.setText(_translate("Dialog", "Password"))
        self.groupBox_81.setTitle(_translate("Dialog", "LoTW"))
        self.label_22.setText(_translate("Dialog", "Username"))
        self.lotwUserLineEdit.setToolTip(_translate("Dialog", "Username for LoTW online account not for the certificate"))
        self.label_211.setText(_translate("Dialog", "Password"))
        self.lotwPasswdLineEdit.setToolTip(_translate("Dialog", "Password for LoTW online account not for the certificate"))
        self.lotwCertPwdCheckBox.setText(_translate("Dialog", "Certificate needs password"))
        self.label_23.setText(_translate("Dialog", "TQSL"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.credentials), _translate("Dialog", "Credentials"))



from PyQt6 import QtWidgets, QtCore

from . import ListEdit_ui


class ListEdit(QtWidgets.QWidget, ListEdit_ui.Ui_ListEditForm):
    listChanged = QtCore.pyqtSignal()

    def __init__(self, parent, titel=''):
        super().__init__(parent)
        self.setupUi(self)

        if not titel:
            self.layout().removeWidget(self.label)
            del self.label
        else:
            self.label.setText(titel)

        self.listWidget.itemChanged.connect(self.listChanged.emit)

    def clear(self):
        self.listWidget.clear()
        self.delPushButton.setEnabled(False)
        self.listChanged.emit()

    def items(self):
        items = []
        for i in range(self.listWidget.count()):
            if self.listWidget.item(i).text().strip() not in (self.tr('(empty)'), ''):
                items.append(self.listWidget.item(i).text())
        return items

    def setItems(self, items: list[str]):
        if items:
            for item_str in items:
                if item_str:
                    item = QtWidgets.QListWidgetItem(item_str)
                    item.setFlags(item.flags() | QtCore.Qt.ItemFlag.ItemIsEditable)
                    self.listWidget.addItem(item)
                    self.delPushButton.setEnabled(True)
                    self.listChanged.emit()

    def addEmptyItem(self):
        item = QtWidgets.QListWidgetItem(self.tr('(empty)'))
        item.setFlags(item.flags() | QtCore.Qt.ItemFlag.ItemIsEditable)
        self.listWidget.addItem(item)
        self.delPushButton.setEnabled(True)

    def removeSelectedItem(self):
        for item in self.listWidget.selectedItems():
            self.listWidget.takeItem(self.listWidget.row(item))
            self.listChanged.emit()

        if self.listWidget.count() < 1:
            self.delPushButton.setEnabled(False)



# DragonLog (c) 2025 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/
"""Writes a file with list of all available contests as Markdown if a file name is given.
Else prints out the content."""

import sys

from . import build_contest_list

if len(sys.argv) > 1:
    with open(sys.argv[1], 'w', encoding='UTF-8') as cf:
        cf.write(build_contest_list())
else:
    print(build_contest_list())



# Form implementation generated from reading ui file 'CassiopeiaConsole.ui'
#
# Created by: PyQt6 UI code generator 6.9.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_CassiopeiaConsoleWidget(object):
    def setupUi(self, CassiopeiaConsoleWidget):
        CassiopeiaConsoleWidget.setObjectName("CassiopeiaConsoleWidget")
        CassiopeiaConsoleWidget.resize(800, 227)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(CassiopeiaConsoleWidget.sizePolicy().hasHeightForWidth())
        CassiopeiaConsoleWidget.setSizePolicy(sizePolicy)
        CassiopeiaConsoleWidget.setMinimumSize(QtCore.QSize(800, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(CassiopeiaConsoleWidget)
        self.verticalLayout.setContentsMargins(-1, 6, -1, 6)
        self.verticalLayout.setSpacing(3)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(10)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(parent=CassiopeiaConsoleWidget)
        self.groupBox_2.setStyleSheet("QGroupBox {background-color: rgba(0, 0, 255, 63)}")
        self.groupBox_2.setObjectName("groupBox_2")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.groupBox_2)
        self.horizontalLayout_4.setContentsMargins(6, 0, 6, 3)
        self.horizontalLayout_4.setSpacing(9)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_2 = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)
        self.myCallLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.myCallLineEdit.sizePolicy().hasHeightForWidth())
        self.myCallLineEdit.setSizePolicy(sizePolicy)
        self.myCallLineEdit.setProperty("cssClass", "CCInput")
        self.myCallLineEdit.setObjectName("myCallLineEdit")
        self.horizontalLayout_4.addWidget(self.myCallLineEdit)
        self.line_6 = QtWidgets.QFrame(parent=self.groupBox_2)
        self.line_6.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_6.setObjectName("line_6")
        self.horizontalLayout_4.addWidget(self.line_6)
        self.label_10 = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_4.addWidget(self.label_10)
        self.myLocLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(3)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.myLocLineEdit.sizePolicy().hasHeightForWidth())
        self.myLocLineEdit.setSizePolicy(sizePolicy)
        self.myLocLineEdit.setProperty("cssClass", "CCInput")
        self.myLocLineEdit.setObjectName("myLocLineEdit")
        self.horizontalLayout_4.addWidget(self.myLocLineEdit)
        self.line_7 = QtWidgets.QFrame(parent=self.groupBox_2)
        self.line_7.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_7.setObjectName("line_7")
        self.horizontalLayout_4.addWidget(self.line_7)
        self.label_11 = QtWidgets.QLabel(parent=self.groupBox_2)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_4.addWidget(self.label_11)
        self.myNameLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.myNameLineEdit.sizePolicy().hasHeightForWidth())
        self.myNameLineEdit.setSizePolicy(sizePolicy)
        self.myNameLineEdit.setProperty("cssClass", "CCInput")
        self.myNameLineEdit.setObjectName("myNameLineEdit")
        self.horizontalLayout_4.addWidget(self.myNameLineEdit)
        self.horizontalLayout.addWidget(self.groupBox_2)
        self.groupBox_3 = QtWidgets.QGroupBox(parent=CassiopeiaConsoleWidget)
        self.groupBox_3.setStyleSheet("QGroupBox {background-color: rgba(127, 0, 255, 63)}")
        self.groupBox_3.setObjectName("groupBox_3")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.groupBox_3)
        self.horizontalLayout_8.setContentsMargins(6, 0, 6, 3)
        self.horizontalLayout_8.setSpacing(9)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_19 = QtWidgets.QLabel(parent=self.groupBox_3)
        self.label_19.setObjectName("label_19")
        self.horizontalLayout_8.addWidget(self.label_19)
        self.eventComboBox = QtWidgets.QComboBox(parent=self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.MinimumExpanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(3)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.eventComboBox.sizePolicy().hasHeightForWidth())
        self.eventComboBox.setSizePolicy(sizePolicy)
        self.eventComboBox.setProperty("cssClass", "CCInput")
        self.eventComboBox.setObjectName("eventComboBox")
        self.eventComboBox.addItem("")
        self.eventComboBox.addItem("")
        self.eventComboBox.addItem("")
        self.horizontalLayout_8.addWidget(self.eventComboBox)
        self.line_15 = QtWidgets.QFrame(parent=self.groupBox_3)
        self.line_15.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_15.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_15.setObjectName("line_15")
        self.horizontalLayout_8.addWidget(self.line_15)
        self.label_21 = QtWidgets.QLabel(parent=self.groupBox_3)
        self.label_21.setObjectName("label_21")
        self.horizontalLayout_8.addWidget(self.label_21)
        self.exchTXLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.exchTXLineEdit.sizePolicy().hasHeightForWidth())
        self.exchTXLineEdit.setSizePolicy(sizePolicy)
        self.exchTXLineEdit.setProperty("cssClass", "CCInput")
        self.exchTXLineEdit.setObjectName("exchTXLineEdit")
        self.horizontalLayout_8.addWidget(self.exchTXLineEdit)
        self.label = QtWidgets.QLabel(parent=self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout_8.addWidget(self.label)
        self.line_16 = QtWidgets.QFrame(parent=self.groupBox_3)
        self.line_16.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_16.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_16.setObjectName("line_16")
        self.horizontalLayout_8.addWidget(self.line_16)
        self.label_20 = QtWidgets.QLabel(parent=self.groupBox_3)
        self.label_20.setObjectName("label_20")
        self.horizontalLayout_8.addWidget(self.label_20)
        self.exchRXLineEdit = QtWidgets.QLineEdit(parent=self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.exchRXLineEdit.sizePolicy().hasHeightForWidth())
        self.exchRXLineEdit.setSizePolicy(sizePolicy)
        self.exchRXLineEdit.setProperty("cssClass", "CCInput")
        self.exchRXLineEdit.setObjectName("exchRXLineEdit")
        self.horizontalLayout_8.addWidget(self.exchRXLineEdit)
        self.label_6 = QtWidgets.QLabel(parent=self.groupBox_3)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_8.addWidget(self.label_6)
        self.horizontalLayout.addWidget(self.groupBox_3)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.qsoDataGroupBox = QtWidgets.QGroupBox(parent=CassiopeiaConsoleWidget)
        self.qsoDataGroupBox.setStyleSheet("QGroupBox {background-color: rgba(255, 255, 0, 63)}")
        self.qsoDataGroupBox.setObjectName("qsoDataGroupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.qsoDataGroupBox)
        self.verticalLayout_2.setContentsMargins(6, 0, 6, 3)
        self.verticalLayout_2.setSpacing(3)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setSpacing(9)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.dateEdit = QtWidgets.QDateEdit(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dateEdit.sizePolicy().hasHeightForWidth())
        self.dateEdit.setSizePolicy(sizePolicy)
        self.dateEdit.setCalendarPopup(True)
        self.dateEdit.setProperty("cssClass", "CCInput")
        self.dateEdit.setObjectName("dateEdit")
        self.horizontalLayout_5.addWidget(self.dateEdit)
        self.label_3 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_5.addWidget(self.label_3)
        self.line_3 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_3.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout_5.addWidget(self.line_3)
        self.timeEdit = QtWidgets.QTimeEdit(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeEdit.sizePolicy().hasHeightForWidth())
        self.timeEdit.setSizePolicy(sizePolicy)
        self.timeEdit.setProperty("cssClass", "CCInput")
        self.timeEdit.setObjectName("timeEdit")
        self.horizontalLayout_5.addWidget(self.timeEdit)
        self.label_4 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.line_2 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_2.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_5.addWidget(self.line_2)
        self.bandComboBox = QtWidgets.QComboBox(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bandComboBox.sizePolicy().hasHeightForWidth())
        self.bandComboBox.setSizePolicy(sizePolicy)
        self.bandComboBox.setProperty("cssClass", "CCInput")
        self.bandComboBox.setObjectName("bandComboBox")
        self.bandComboBox.addItem("")
        self.horizontalLayout_5.addWidget(self.bandComboBox)
        self.line = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_5.addWidget(self.line)
        self.modeComboBox = QtWidgets.QComboBox(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.modeComboBox.sizePolicy().hasHeightForWidth())
        self.modeComboBox.setSizePolicy(sizePolicy)
        self.modeComboBox.setProperty("cssClass", "CCInput")
        self.modeComboBox.setObjectName("modeComboBox")
        self.modeComboBox.addItem("")
        self.horizontalLayout_5.addWidget(self.modeComboBox)
        self.line_4 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_4.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout_5.addWidget(self.line_4)
        self.callLineEdit = QtWidgets.QLineEdit(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.callLineEdit.sizePolicy().hasHeightForWidth())
        self.callLineEdit.setSizePolicy(sizePolicy)
        self.callLineEdit.setProperty("cssClass", "CCInput")
        self.callLineEdit.setObjectName("callLineEdit")
        self.horizontalLayout_5.addWidget(self.callLineEdit)
        self.line_8 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_8.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_8.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_8.setObjectName("line_8")
        self.horizontalLayout_5.addWidget(self.line_8)
        self.label_9 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_5.addWidget(self.label_9)
        self.locLineEdit = QtWidgets.QLineEdit(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(3)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.locLineEdit.sizePolicy().hasHeightForWidth())
        self.locLineEdit.setSizePolicy(sizePolicy)
        self.locLineEdit.setProperty("cssClass", "CCInput")
        self.locLineEdit.setObjectName("locLineEdit")
        self.horizontalLayout_5.addWidget(self.locLineEdit)
        self.line_9 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_9.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_9.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_9.setObjectName("line_9")
        self.horizontalLayout_5.addWidget(self.line_9)
        self.label_13 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_5.addWidget(self.label_13)
        self.rstTXLineEdit = QtWidgets.QLineEdit(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.rstTXLineEdit.sizePolicy().hasHeightForWidth())
        self.rstTXLineEdit.setSizePolicy(sizePolicy)
        self.rstTXLineEdit.setMaximumSize(QtCore.QSize(50, 16777215))
        self.rstTXLineEdit.setProperty("cssClass", "CCInput")
        self.rstTXLineEdit.setObjectName("rstTXLineEdit")
        self.horizontalLayout_5.addWidget(self.rstTXLineEdit)
        self.label_7 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_5.addWidget(self.label_7)
        self.line_5 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_5.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_5.setObjectName("line_5")
        self.horizontalLayout_5.addWidget(self.line_5)
        self.label_12 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_5.addWidget(self.label_12)
        self.rstRXLineEdit = QtWidgets.QLineEdit(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.rstRXLineEdit.sizePolicy().hasHeightForWidth())
        self.rstRXLineEdit.setSizePolicy(sizePolicy)
        self.rstRXLineEdit.setMaximumSize(QtCore.QSize(50, 16777215))
        self.rstRXLineEdit.setProperty("cssClass", "CCInput")
        self.rstRXLineEdit.setObjectName("rstRXLineEdit")
        self.horizontalLayout_5.addWidget(self.rstRXLineEdit)
        self.label_8 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_5.addWidget(self.label_8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.line_10 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_10.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_10.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_10.setObjectName("line_10")
        self.verticalLayout_2.addWidget(self.line_10)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(9)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_14 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_2.addWidget(self.label_14)
        self.nameLineEdit = QtWidgets.QLineEdit(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nameLineEdit.sizePolicy().hasHeightForWidth())
        self.nameLineEdit.setSizePolicy(sizePolicy)
        self.nameLineEdit.setMaximumSize(QtCore.QSize(200, 16777215))
        self.nameLineEdit.setProperty("cssClass", "CCInput")
        self.nameLineEdit.setObjectName("nameLineEdit")
        self.horizontalLayout_2.addWidget(self.nameLineEdit)
        self.line_14 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_14.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_14.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_14.setObjectName("line_14")
        self.horizontalLayout_2.addWidget(self.line_14)
        self.freqSpinBox = QtWidgets.QSpinBox(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.freqSpinBox.sizePolicy().hasHeightForWidth())
        self.freqSpinBox.setSizePolicy(sizePolicy)
        self.freqSpinBox.setMaximum(999999999)
        self.freqSpinBox.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.freqSpinBox.setProperty("value", 0)
        self.freqSpinBox.setProperty("cssClass", "CCInput")
        self.freqSpinBox.setObjectName("freqSpinBox")
        self.horizontalLayout_2.addWidget(self.freqSpinBox)
        self.label_16 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_16.setObjectName("label_16")
        self.horizontalLayout_2.addWidget(self.label_16)
        self.line_12 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_12.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_12.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_12.setObjectName("line_12")
        self.horizontalLayout_2.addWidget(self.line_12)
        self.powerSpinBox = QtWidgets.QSpinBox(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.powerSpinBox.sizePolicy().hasHeightForWidth())
        self.powerSpinBox.setSizePolicy(sizePolicy)
        self.powerSpinBox.setMinimum(0)
        self.powerSpinBox.setMaximum(9999)
        self.powerSpinBox.setStepType(QtWidgets.QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.powerSpinBox.setProperty("value", 0)
        self.powerSpinBox.setProperty("cssClass", "CCInput")
        self.powerSpinBox.setObjectName("powerSpinBox")
        self.horizontalLayout_2.addWidget(self.powerSpinBox)
        self.label_17 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_17.setObjectName("label_17")
        self.horizontalLayout_2.addWidget(self.label_17)
        self.line_13 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_13.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_13.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_13.setObjectName("line_13")
        self.horizontalLayout_2.addWidget(self.line_13)
        self.label_18 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_18.setObjectName("label_18")
        self.horizontalLayout_2.addWidget(self.label_18)
        self.qslCheckBox = QtWidgets.QCheckBox(parent=self.qsoDataGroupBox)
        font = QtGui.QFont()
        font.setItalic(True)
        self.qslCheckBox.setFont(font)
        self.qslCheckBox.setObjectName("qslCheckBox")
        self.horizontalLayout_2.addWidget(self.qslCheckBox)
        self.line_11 = QtWidgets.QFrame(parent=self.qsoDataGroupBox)
        self.line_11.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_11.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_11.setObjectName("line_11")
        self.horizontalLayout_2.addWidget(self.line_11)
        self.label_15 = QtWidgets.QLabel(parent=self.qsoDataGroupBox)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_2.addWidget(self.label_15)
        self.commentLineEdit = QtWidgets.QLineEdit(parent=self.qsoDataGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(3)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.commentLineEdit.sizePolicy().hasHeightForWidth())
        self.commentLineEdit.setSizePolicy(sizePolicy)
        self.commentLineEdit.setProperty("cssClass", "CCInput")
        self.commentLineEdit.setObjectName("commentLineEdit")
        self.horizontalLayout_2.addWidget(self.commentLineEdit)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addWidget(self.qsoDataGroupBox)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setContentsMargins(6, -1, -1, -1)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_5 = QtWidgets.QLabel(parent=CassiopeiaConsoleWidget)
        self.label_5.setStyleSheet("background-color: rgba(0, 255, 0, 0)")
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_7.addWidget(self.label_5)
        self.inputLineEdit = QtWidgets.QLineEdit(parent=CassiopeiaConsoleWidget)
        self.inputLineEdit.setProperty("cssClass", "CCInput")
        self.inputLineEdit.setObjectName("inputLineEdit")
        self.horizontalLayout_7.addWidget(self.inputLineEdit)
        self.clearPushButton = QtWidgets.QPushButton(parent=CassiopeiaConsoleWidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("icons:no.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.clearPushButton.setIcon(icon)
        self.clearPushButton.setObjectName("clearPushButton")
        self.horizontalLayout_7.addWidget(self.clearPushButton)
        self.addQSOPushButton = QtWidgets.QPushButton(parent=CassiopeiaConsoleWidget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("icons:edit_add.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.addQSOPushButton.setIcon(icon1)
        self.addQSOPushButton.setObjectName("addQSOPushButton")
        self.horizontalLayout_7.addWidget(self.addQSOPushButton)
        self.onlineCheckBox = QtWidgets.QCheckBox(parent=CassiopeiaConsoleWidget)
        self.onlineCheckBox.setObjectName("onlineCheckBox")
        self.horizontalLayout_7.addWidget(self.onlineCheckBox)
        self.verticalLayout_4.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setContentsMargins(6, -1, 6, -1)
        self.horizontalLayout_3.setSpacing(9)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.ctyCtyLabel = QtWidgets.QLabel(parent=CassiopeiaConsoleWidget)
        self.ctyCtyLabel.setText("")
        self.ctyCtyLabel.setObjectName("ctyCtyLabel")
        self.horizontalLayout_3.addWidget(self.ctyCtyLabel)
        self.line_18 = QtWidgets.QFrame(parent=CassiopeiaConsoleWidget)
        self.line_18.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_18.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_18.setObjectName("line_18")
        self.horizontalLayout_3.addWidget(self.line_18)
        self.ctyAreaLabel = QtWidgets.QLabel(parent=CassiopeiaConsoleWidget)
        self.ctyAreaLabel.setText("")
        self.ctyAreaLabel.setObjectName("ctyAreaLabel")
        self.horizontalLayout_3.addWidget(self.ctyAreaLabel)
        self.line_17 = QtWidgets.QFrame(parent=CassiopeiaConsoleWidget)
        self.line_17.setFrameShape(QtWidgets.QFrame.Shape.VLine)
        self.line_17.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_17.setObjectName("line_17")
        self.horizontalLayout_3.addWidget(self.line_17)
        self.distLabel = QtWidgets.QLabel(parent=CassiopeiaConsoleWidget)
        self.distLabel.setText("")
        self.distLabel.setObjectName("distLabel")
        self.horizontalLayout_3.addWidget(self.distLabel)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.resultWidget = QtWidgets.QWidget(parent=CassiopeiaConsoleWidget)
        self.resultWidget.setStyleSheet("#resultWidget {background-color: rgba(0, 255, 0, 63)}")
        self.resultWidget.setObjectName("resultWidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.resultWidget)
        self.verticalLayout_3.setContentsMargins(6, 3, 6, 3)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.resultLabel = QtWidgets.QLabel(parent=self.resultWidget)
        self.resultLabel.setText("")
        self.resultLabel.setWordWrap(True)
        self.resultLabel.setObjectName("resultLabel")
        self.verticalLayout_3.addWidget(self.resultLabel)
        self.verticalLayout_4.addWidget(self.resultWidget)
        self.horizontalLayout_6.addLayout(self.verticalLayout_4)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        spacerItem1 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.verticalLayout.addItem(spacerItem1)

        self.retranslateUi(CassiopeiaConsoleWidget)
        self.eventComboBox.setCurrentIndex(0)
        self.bandComboBox.setCurrentIndex(0)
        self.modeComboBox.setCurrentIndex(0)
        self.clearPushButton.clicked.connect(CassiopeiaConsoleWidget.clearInput) # type: ignore
        self.inputLineEdit.textEdited['QString'].connect(CassiopeiaConsoleWidget.evaluate) # type: ignore
        self.inputLineEdit.returnPressed.connect(CassiopeiaConsoleWidget.finaliseQSO) # type: ignore
        self.addQSOPushButton.clicked.connect(CassiopeiaConsoleWidget.finaliseQSO) # type: ignore
        self.bandComboBox.currentTextChanged['QString'].connect(CassiopeiaConsoleWidget.bandChanged) # type: ignore
        self.dateEdit.userDateChanged['QDate'].connect(CassiopeiaConsoleWidget.dateChanged) # type: ignore
        self.timeEdit.timeChanged['QTime'].connect(CassiopeiaConsoleWidget.timeChanged) # type: ignore
        self.freqSpinBox.valueChanged['int'].connect(CassiopeiaConsoleWidget.freqChanged) # type: ignore
        self.powerSpinBox.valueChanged['int'].connect(CassiopeiaConsoleWidget.powerChanged) # type: ignore
        self.modeComboBox.currentTextChanged['QString'].connect(CassiopeiaConsoleWidget.modeChanged) # type: ignore
        self.callLineEdit.textEdited['QString'].connect(CassiopeiaConsoleWidget.callChanged) # type: ignore
        self.commentLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.commentChanged) # type: ignore
        self.nameLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.nameChanged) # type: ignore
        self.locLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.locatorChanged) # type: ignore
        self.qslCheckBox.clicked['bool'].connect(CassiopeiaConsoleWidget.qslChanged) # type: ignore
        self.rstRXLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.rstRxChanged) # type: ignore
        self.rstTXLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.rstTxChanged) # type: ignore
        self.eventComboBox.currentTextChanged['QString'].connect(CassiopeiaConsoleWidget.eventChanged) # type: ignore
        self.exchRXLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.exchRxChanged) # type: ignore
        self.exchTXLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.exchTxChanged) # type: ignore
        self.myCallLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.myCallChanged) # type: ignore
        self.myNameLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.myNameChanged) # type: ignore
        self.myLocLineEdit.editingFinished.connect(CassiopeiaConsoleWidget.myLocChanged) # type: ignore
        self.onlineCheckBox.clicked['bool'].connect(CassiopeiaConsoleWidget.onlineChanged) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(CassiopeiaConsoleWidget)
        CassiopeiaConsoleWidget.setTabOrder(self.inputLineEdit, self.myCallLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.myCallLineEdit, self.myLocLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.myLocLineEdit, self.myNameLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.myNameLineEdit, self.exchTXLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.exchTXLineEdit, self.exchRXLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.exchRXLineEdit, self.callLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.callLineEdit, self.locLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.locLineEdit, self.rstTXLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.rstTXLineEdit, self.nameLineEdit)
        CassiopeiaConsoleWidget.setTabOrder(self.nameLineEdit, self.commentLineEdit)

    def retranslateUi(self, CassiopeiaConsoleWidget):
        _translate = QtCore.QCoreApplication.translate
        CassiopeiaConsoleWidget.setWindowTitle(_translate("CassiopeiaConsoleWidget", "CassiopeiaConsole"))
        self.groupBox_2.setTitle(_translate("CassiopeiaConsoleWidget", "Your Data"))
        self.label_2.setText(_translate("CassiopeiaConsoleWidget", "-c"))
        self.myCallLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Your callsign"))
        self.label_10.setText(_translate("CassiopeiaConsoleWidget", "-l"))
        self.myLocLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Your Locator"))
        self.label_11.setText(_translate("CassiopeiaConsoleWidget", "-n"))
        self.myNameLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Your Name"))
        self.groupBox_3.setTitle(_translate("CassiopeiaConsoleWidget", "Event"))
        self.label_19.setText(_translate("CassiopeiaConsoleWidget", "$"))
        self.eventComboBox.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Event ID"))
        self.eventComboBox.setItemText(0, _translate("CassiopeiaConsoleWidget", "Event ID"))
        self.eventComboBox.setItemText(1, _translate("CassiopeiaConsoleWidget", "POTA"))
        self.eventComboBox.setItemText(2, _translate("CassiopeiaConsoleWidget", "SOTA"))
        self.label_21.setText(_translate("CassiopeiaConsoleWidget", "-N"))
        self.exchTXLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Exchange"))
        self.label.setText(_translate("CassiopeiaConsoleWidget", "Tx"))
        self.label_20.setText(_translate("CassiopeiaConsoleWidget", "%"))
        self.exchRXLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Exchange"))
        self.label_6.setText(_translate("CassiopeiaConsoleWidget", "Rx"))
        self.qsoDataGroupBox.setTitle(_translate("CassiopeiaConsoleWidget", "QSO Data"))
        self.dateEdit.setDisplayFormat(_translate("CassiopeiaConsoleWidget", "yyyy-MM-dd"))
        self.label_3.setText(_translate("CassiopeiaConsoleWidget", "d"))
        self.label_4.setText(_translate("CassiopeiaConsoleWidget", "t"))
        self.bandComboBox.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Band"))
        self.bandComboBox.setItemText(0, _translate("CassiopeiaConsoleWidget", "Band"))
        self.modeComboBox.setCurrentText(_translate("CassiopeiaConsoleWidget", "Mode"))
        self.modeComboBox.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Mode"))
        self.modeComboBox.setItemText(0, _translate("CassiopeiaConsoleWidget", "Mode"))
        self.callLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Call"))
        self.label_9.setText(_translate("CassiopeiaConsoleWidget", "@"))
        self.locLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Locator"))
        self.label_13.setText(_translate("CassiopeiaConsoleWidget", ","))
        self.rstTXLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "RST"))
        self.label_7.setText(_translate("CassiopeiaConsoleWidget", "Tx"))
        self.label_12.setText(_translate("CassiopeiaConsoleWidget", "."))
        self.rstRXLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "RST"))
        self.label_8.setText(_translate("CassiopeiaConsoleWidget", "Rx"))
        self.label_14.setText(_translate("CassiopeiaConsoleWidget", "\'"))
        self.nameLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Name"))
        self.freqSpinBox.setSpecialValueText(_translate("CassiopeiaConsoleWidget", "n.a."))
        self.freqSpinBox.setSuffix(_translate("CassiopeiaConsoleWidget", " kHz"))
        self.label_16.setText(_translate("CassiopeiaConsoleWidget", "f"))
        self.powerSpinBox.setSpecialValueText(_translate("CassiopeiaConsoleWidget", "n.a."))
        self.powerSpinBox.setSuffix(_translate("CassiopeiaConsoleWidget", " W"))
        self.label_17.setText(_translate("CassiopeiaConsoleWidget", "p"))
        self.label_18.setText(_translate("CassiopeiaConsoleWidget", "*"))
        self.qslCheckBox.setText(_translate("CassiopeiaConsoleWidget", "QSL rcvd"))
        self.label_15.setText(_translate("CassiopeiaConsoleWidget", "#"))
        self.commentLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Comment"))
        self.label_5.setText(_translate("CassiopeiaConsoleWidget", "QSO>"))
        self.inputLineEdit.setPlaceholderText(_translate("CassiopeiaConsoleWidget", "Input"))
        self.clearPushButton.setToolTip(_translate("CassiopeiaConsoleWidget", "Clear"))
        self.addQSOPushButton.setToolTip(_translate("CassiopeiaConsoleWidget", "Push to Log"))
        self.onlineCheckBox.setText(_translate("CassiopeiaConsoleWidget", "Online"))



# DragonLog (c) 2025 by Andreas Schawo is licensed under CC BY-SA 4.0.
# To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/
# based on work of TobbY, DG1ATN

import logging

from PyQt6 import QtWidgets, QtCore, QtGui
# noinspection PyPackageRequirements
import cv2
from pyzbar import pyzbar
from adif_file import util

from . import QSLQRReader_ui
from .Logger import Logger


class QSLQRReaderDialog(QtWidgets.QDialog, QSLQRReader_ui.Ui_QSLQRReaderDialog):
    qslQrReceived = QtCore.pyqtSignal(list)

    def __init__(self, parent, dragonlog, settings: QtCore.QSettings, logger: Logger):
        super().__init__(parent)
        self.dragonlog = dragonlog
        self.setupUi(self)

        self.log = logging.getLogger('QSLQRReaderDialog')
        self.log.addHandler(logger)
        self.log.setLevel(logger.loglevel)
        self.logger = logger
        self.log.debug('Initialising...')

        self.setWindowTitle(self.tr('QSL-QR-Reader'))

        self.__settings__ = settings
        self.__video_thread__ = None

        self.qslQrReceived.connect(self.processQSL)

        self.updateDeviceList()

    def updateDeviceList(self):
        self.videoSrcComboBox.clear()
        devices = []
        for i in range(10):
            cap = cv2.VideoCapture(i)
            test, frame = cap.read()
            if test:
                devices.append(i)
                cap.release()
        self.videoSrcComboBox.insertItems(0, [f'Video #{d}' for d in devices])

    def stopCapturing(self):
        self.log.debug(f'Stopping capture...')
        if self.__video_thread__:
            self.__video_thread__.stop()
            self.__video_thread__ = None

    def videoSrcChanged(self, src: str):
        self.stopCapturing()
        self.log.debug(f'Capturing from {src}')
        self.__video_thread__ = CaptureThread(None, int(src[-1:]))
        self.__video_thread__.pixmapChanged.connect(self.displayImage)
        self.__video_thread__.qrReceived.connect(self.processQSLData)
        self.__video_thread__.codeTypeFound.connect(self.displayMessage)
        self.__video_thread__.start()

    def displayImage(self, image: QtGui.QPixmap):
        self.displayLabel.setPixmap(image)

    def displayMessage(self, msg: str):
        self.msgLabel.setText(str(msg))
        QtCore.QTimer.singleShot(3000, self.msgLabel.clear)

    def processQSLData(self, data: str):
        """Process the raw QR code data"""
        records = []
        call = ''
        own_call = ''
        rec = {}
        try:
            data = data.replace(': ', ';')
            for i, ln in enumerate(data.split('\n')):
                if i == 0:
                    if ln.startswith('From') and 'To' in ln:
                        call_kv, own_call_kv = ln.split(' ')
                        call = call_kv.split(';')[1]
                        own_call = own_call_kv.split(';')[1]
                    else:
                        raise ValueError('No valid QSL-QR-Code')
                else:
                    if ln.startswith('Date'):
                        if rec:
                            records.append(rec)

                        rec = {'CALL': call,
                               'STATION_CALL': own_call,
                               'QSL_RCVD': 'Y',
                               }

                        for kv in ln.split(' '):
                            try:
                                k, v = kv.split(';')
                                if k == 'Date':
                                    rec['QSO_DATE'] = f'20{v[6:]}{v[3:5]}{v[:2]}'
                                elif k == 'Time':
                                    rec['TIME_ON'] = v.replace(':', '')
                                elif k == 'Band':
                                    if not 'BAND' in rec:
                                        rec['BAND'] = v
                                    rec['BAND_RX'] = v
                                elif k == 'Band_RX':
                                    rec['BAND'] = v
                                elif k == 'Mode':
                                    rec['MODE'] = v
                                elif k == 'RST':
                                    rec['RST_RCVD'] = v
                                elif k == 'QSL':
                                    if v == 'PSE':
                                        rec['QSL_SENT'] = 'R'  # requested
                                    elif v == 'TNX':
                                        rec['QSL_SENT'] = 'Y'  # assume we sent QSL
                            except ValueError:
                                # if key/value can't be split
                                pass
                    elif ln.startswith('Comment'):
                        try:
                            rec['COMMENT'] = ln.split(';', 1)[1]
                        except IndexError:
                            pass
                    elif not ln.strip() and rec:
                        records.append(rec)
        except (ValueError, IndexError):
            self.log.warning('No valid QSL-QR-Code')
            self.log.debug(data.strip())
            self.displayMessage(self.tr('No valid QSL-QR-Code'))
            self.videoSrcChanged(self.videoSrcComboBox.currentText())

        if records:
            self.displayMessage(self.tr('Found valid QSL-QR-Code'))
            self.qslQrReceived.emit(records)

    def processQSL(self, data: list[dict]):
        """Process the resulting QSL"""

        report = []
        for i, qsl in enumerate(data):
            timestamp = util.adif_date2iso(qsl['QSO_DATE']) + ' ' + util.adif_time2iso(qsl['TIME_ON'])
            qso = self.dragonlog.findQSO(timestamp, qsl['CALL'], 5)

            if qso:
                # Update QSL sent anyway
                cur_qsl_sent = qso[self.dragonlog.__sql_cols__.index('qsl_sent')]
                if cur_qsl_sent != 'Y' and qsl['QSL_SENT'] == 'R':
                    self.dragonlog.updateQSOField('qsl_sent', qso[0], 'R')
                    report.append(f'QSL #{i + 1}: {self.tr("Marked QSL sent as requested for QSO ")} #{qso[0]}')

                # Skip QSLed QSO or mark QSL rcvd
                cur_qsl_rcvd = qso[self.dragonlog.__sql_cols__.index('qsl_rcvd')]
                if cur_qsl_rcvd == 'Y':
                    report.append(f'QSL #{i + 1}: {self.tr("QSL already marked as received for QSO ")} #{qso[0]}')
                    continue
                self.dragonlog.updateQSOField('qsl_rcvd', qso[0], 'Y')
                report.append(f'QSL #{i + 1}: {self.tr("Marked QSL as received for QSO ")} #{qso[0]}')
            else:
                self.dragonlog.logImportQRCode(qsl)
                report.append(f'QSL #{i + 1}: ' + self.tr('Imported QSO to logbook'))

        if report:
            QtWidgets.QMessageBox.information(self, self.tr('QSL-QR-Code import'),
                                              '\n'.join(report))
            self.log.info(', '.join(report))
        self.videoSrcChanged(self.videoSrcComboBox.currentText())

    def accept(self):
        self.stopCapturing()
        super().accept()

    def reject(self):
        self.stopCapturing()
        super().reject()


class CaptureThread(QtCore.QThread):
    pixmapChanged = QtCore.pyqtSignal(QtGui.QPixmap)
    qrReceived = QtCore.pyqtSignal(str)
    codeTypeFound = QtCore.pyqtSignal(str)

    def __init__(self, parent, device):
        super().__init__(parent)
        self.device_id = device
        self.__run__ = True

    def run(self):
        cap = cv2.VideoCapture(self.device_id)
        while self.__run__:
            ret, cv_img = cap.read()
            if ret:
                self.pixmapChanged.emit(self.convertImage(cv_img))
                barcode = pyzbar.decode(cv_img)
                if barcode:
                    self.codeTypeFound.emit(self.tr('Found code type') + ': ' + barcode[0].type)
                    if barcode[0].type == 'QRCODE':
                        self.__run__ = False
                        self.qrReceived.emit(barcode[0].data.decode())
        cap.release()

    def stop(self):
        self.__run__ = False
        self.wait()

    @staticmethod
    def convertImage(cv_img):
        """Convert from an opencv image to QPixmap"""
        rgb_image = cv2.cvtColor(cv_img, cv2.COLOR_BGR2RGB)
        h, w, ch = rgb_image.shape
        bytes_per_line = ch * w
        pixmap = QtGui.QImage(bytes(rgb_image.data), w, h, bytes_per_line, QtGui.QImage.Format.Format_RGB888)
        p = pixmap.scaled(480, 360, QtCore.Qt.AspectRatioMode.KeepAspectRatio)
        return QtGui.QPixmap.fromImage(p)



# Form implementation generated from reading ui file 'QSLQRReader.ui'
#
# Created by: PyQt6 UI code generator 6.9.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_QSLQRReaderDialog(object):
    def setupUi(self, QSLQRReaderDialog):
        QSLQRReaderDialog.setObjectName("QSLQRReaderDialog")
        QSLQRReaderDialog.resize(400, 330)
        self.verticalLayout = QtWidgets.QVBoxLayout(QSLQRReaderDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QtWidgets.QFrame(parent=QSLQRReaderDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.frame.setObjectName("frame")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.displayLabel = QtWidgets.QLabel(parent=self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.displayLabel.sizePolicy().hasHeightForWidth())
        self.displayLabel.setSizePolicy(sizePolicy)
        self.displayLabel.setText("")
        self.displayLabel.setObjectName("displayLabel")
        self.verticalLayout_2.addWidget(self.displayLabel)
        self.verticalLayout.addWidget(self.frame)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.srcLabel = QtWidgets.QLabel(parent=QSLQRReaderDialog)
        self.srcLabel.setObjectName("srcLabel")
        self.horizontalLayout_2.addWidget(self.srcLabel)
        self.videoSrcComboBox = QtWidgets.QComboBox(parent=QSLQRReaderDialog)
        self.videoSrcComboBox.setObjectName("videoSrcComboBox")
        self.horizontalLayout_2.addWidget(self.videoSrcComboBox)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.msgLabel = QtWidgets.QLabel(parent=QSLQRReaderDialog)
        self.msgLabel.setText("")
        self.msgLabel.setObjectName("msgLabel")
        self.verticalLayout.addWidget(self.msgLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.okPushButton = QtWidgets.QPushButton(parent=QSLQRReaderDialog)
        self.okPushButton.setObjectName("okPushButton")
        self.horizontalLayout.addWidget(self.okPushButton)
        self.cancelPushButton = QtWidgets.QPushButton(parent=QSLQRReaderDialog)
        self.cancelPushButton.setObjectName("cancelPushButton")
        self.horizontalLayout.addWidget(self.cancelPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(QSLQRReaderDialog)
        self.okPushButton.clicked.connect(QSLQRReaderDialog.accept) # type: ignore
        self.cancelPushButton.clicked.connect(QSLQRReaderDialog.reject) # type: ignore
        self.videoSrcComboBox.currentTextChanged['QString'].connect(QSLQRReaderDialog.videoSrcChanged) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(QSLQRReaderDialog)

    def retranslateUi(self, QSLQRReaderDialog):
        _translate = QtCore.QCoreApplication.translate
        QSLQRReaderDialog.setWindowTitle(_translate("QSLQRReaderDialog", "Dialog"))
        self.srcLabel.setText(_translate("QSLQRReaderDialog", "Video source"))
        self.okPushButton.setText(_translate("QSLQRReaderDialog", "Ok"))
        self.cancelPushButton.setText(_translate("QSLQRReaderDialog", "Cancel"))
